@@ -5,9 +5,11 @@ name = "tqsdk"
 
 from tqsdk.api import TqApi
 from tqsdk.account import TqAccount, TqKq
+from tqsdk.auth import TqAuth
 from tqsdk.channel import TqChan
 from tqsdk.backtest import TqBacktest, TqReplay
-from tqsdk.exceptions import BacktestFinished
-from tqsdk.lib import TargetPosTask, InsertOrderUntilAllTradedTask, InsertOrderTask
+from tqsdk.exceptions import BacktestFinished, TqTimeoutError
+from tqsdk.lib import TargetPosScheduler, TargetPosTask, InsertOrderUntilAllTradedTask, InsertOrderTask
 from tqsdk.sim import TqSim
+from tqsdk.multiaccount import TqMultiAccount
 from .__version__ import __version__
